# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Module containing SizeControl related classes and methods."""
from ansys.meshing.prime.autogen.sizecontrol import SizeControl as _SizeControl

# isort: split
from ansys.meshing.prime.autogen.commonstructs import SetNameResults
from ansys.meshing.prime.autogen.sizecontrolstructs import SizeControlSummaryParams


class SizeControl(_SizeControl):
    """Size control is used to compute the size field.

    The size field is computed based on the size control defined.
    Different type of size controls provide control over how the mesh size is distributed on a
    surface or within the volume.

    Parameters
    ----------
    model : Model
        Server model to create SizeControl object.
    id : int
        Id of the SizeControl.
    object_id : int
        Object id of the SizeControl.
    name : str
        Name of the SizeControl..
    local : bool, optional
        Unused. The default is ``False``.
    """

    def __init__(self, model, id, object_id, name, local=False):
        """Initialize class variables and the superclass."""
        _SizeControl.__init__(self, model, id, object_id, name)
        self._model = model
        self._name = name

    def __str__(self) -> str:
        """Get a representation of the class in string format.

        Returns
        -------
        str
            Class data in string format.
        """
        params = SizeControlSummaryParams(model=self._model)
        result = _SizeControl.get_summary(self, params)
        return result.message

    def set_suggested_name(self, name: str) -> SetNameResults:
        """Set the unique name for the size control to a suggested name.

        Parameters
        ----------
        name : str
            Suggested name for the size control.

        Returns
        -------
        SetNameResults
            Newly suggested name for the size control.


        Examples
        --------
        >>> size_control.set_suggested_name("control1")

        """
        result = _SizeControl.set_suggested_name(self, name)
        self._name = result.assigned_name
        return result

    @property
    def name(self):
        """Get the name of size control.

        Returns
        -------
        str
            Name of the size control.

        Examples
        --------
        >>> print(size_control.name)

        """
        return self._name



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class Morpher(CoreObject):
    """Morpher contain functionalities to change the geometry, adapting the mesh accordingly without a recomputation.

    Parameters
    ----------
    model : Model
        Server model to create Morpher object.
    """

    def __init__(self, model: CommunicationManager):
        """ Initialize Morpher """
        self._model = model
        self._comm = model._communicator
        command_name = "PrimeMesh::Morpher/Construct"
        args = {"ModelID" : model._object_id , "MaxID" : -1 }
        result = self._comm.serve(model, command_name, args=args)
        self._object_id = result["ObjectIndex"]
        self._freeze()

    def __enter__(self):
        """ Enter context for Morpher. """
        return self

    def __exit__(self, type, value, traceback) :
        """ Exit context for Morpher. """
        command_name = "PrimeMesh::Morpher/Destruct"
        self._comm.serve(self._model, command_name, self._object_id, args={})

    def match_morph(self, part_id : int, match_pairs : List[MatchPair], match_morph_params : MatchMorphParams, bc_params : MorphBCParams, solve_params : MorphSolveParams) -> MatchMorphResults:
        """ Matches source and target zonelets defined using match pairs with prescribed boundary conditions for each pair. Also, solves boundary condition parameters to define uniform surface and volume regions in source neighborhood.


        Parameters
        ----------
        part_id : int
            Id of source part.
        match_pairs : MatchPairArray
            Array of  source and target match pairs.
        match_morph_params : MatchMorphParams
            Match morph parameters.
        bc_params : MorphBCParams
            Morph boundary condition parameters.
        solve_params : MorphSolveParams
            Morpher solve parameters.

        Returns
        -------
        MatchMorphResults
            Returns the MatchMorphResults.


        Examples
        --------
        >>> result = morph.match_morph([match_pair1, match_pair2], match_morph_params, bc_params, solve_params)

        """
        if not isinstance(part_id, int):
            raise TypeError("Invalid argument type passed for 'part_id'. Valid argument type is int.")
        if not isinstance(match_pairs, List):
            raise TypeError("Invalid argument type passed for 'match_pairs'. Valid argument type is List[MatchPair].")
        if not isinstance(match_morph_params, MatchMorphParams):
            raise TypeError("Invalid argument type passed for 'match_morph_params'. Valid argument type is MatchMorphParams.")
        if not isinstance(bc_params, MorphBCParams):
            raise TypeError("Invalid argument type passed for 'bc_params'. Valid argument type is MorphBCParams.")
        if not isinstance(solve_params, MorphSolveParams):
            raise TypeError("Invalid argument type passed for 'solve_params'. Valid argument type is MorphSolveParams.")
        args = {"part_id" : part_id,
        "match_pairs" : [p._jsonify() for p in match_pairs],
        "match_morph_params" : match_morph_params._jsonify(),
        "bc_params" : bc_params._jsonify(),
        "solve_params" : solve_params._jsonify()}
        command_name = "PrimeMesh::Morpher/MatchMorph"
        self._model._print_logs_before_command("match_morph", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("match_morph", MatchMorphResults(model = self._model, json_data = result))
        return MatchMorphResults(model = self._model, json_data = result)



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Module for control data."""
from typing import Iterable, List

# isort: split
from ansys.meshing.prime.autogen.controldata import ControlData as _ControlData

# isort: split
from ansys.meshing.prime.autogen.commonstructs import DeleteResults
from ansys.meshing.prime.autogen.model import Model
from ansys.meshing.prime.autogen.multizonecontrol import MultiZoneControl
from ansys.meshing.prime.autogen.primeconfig import ErrorCode
from ansys.meshing.prime.autogen.shellblcontrol import ShellBLControl
from ansys.meshing.prime.autogen.thinvolumecontrol import ThinVolumeControl
from ansys.meshing.prime.core.periodiccontrol import PeriodicControl
from ansys.meshing.prime.core.prismcontrol import PrismControl
from ansys.meshing.prime.core.sizecontrol import SizeControl
from ansys.meshing.prime.core.volumecontrol import VolumeControl
from ansys.meshing.prime.core.wrappercontrol import WrapperControl
from ansys.meshing.prime.params.primestructs import SizingType


class ControlData(_ControlData):
    """Contains all controls.

    This class contains all controls, including size controls, prism controls, multizone controls
    and wrapper controls.

    Parameters
    ----------
    model : Model
        Server model to create ControlData object.
    id : int
        Id of the ControlData.
    object_id : int
        Object id of the ControlData.
    name : str
        Name of the ControlData.
    """

    def __init__(self, model: Model, id: int, object_id: int, name: str):
        """Initialize the ``ControlData`` class."""
        self._model = model
        self._wrapper_controls = []
        self._mz_controls = []
        self._size_controls = []
        self._prism_controls = []
        self._thin_volume_controls = []
        self._volume_controls = []
        self._periodic_controls = []
        self._shell_bl_controls = []
        _ControlData.__init__(self, model, id, object_id, name)

    def get_wrapper_control_by_name(self, name) -> WrapperControl:
        """Get the wrapper control by name.

        Parameters
        ----------
        name : str
            Name of the wrapper control.

        Returns
        -------
        WrapperControl
            Wrapper control.

        Examples
        --------
        >>> wrapper_control = model.control_data.get_wrapper_control_by_name("wrappercontrol-1")

        """
        for wc in self._wrapper_controls:
            if wc.name == name:
                return wc
        return None

    def get_multi_zone_control_by_name(self, name) -> MultiZoneControl:
        """Get the multizone control by name.

        Parameters
        ----------
        name : str
            Name of the multizone control.

        Returns
        -------
        MultiZoneControl
            Returns the multizone control.

        Examples
        --------
        >>> multi_zone_control = model.control_data.get_multi_zone_control_by_name("mzcontrol-1")

        """
        for mc in self._mz_controls:
            if mc.name == name:
                return mc
        return None

    def create_size_control(self, sizing_type: SizingType) -> SizeControl:
        """Create a size control for a sizing type.

        Parameters
        ----------
        type : SizingType
            Sizing type for creating the size control.

        Returns
        -------
        SizeControl
            Size control.

        Notes
        -----
        An empty size control is created on calling this method.

        Examples
        --------
        >>> size_control = model.control_data.create_size_control(SizingType.Curvature)

        """
        res = _ControlData.create_size_control(self, sizing_type)
        new_size_control = SizeControl(self._model, res[0], res[1], res[2])
        self._size_controls.append(new_size_control)
        return new_size_control

    def create_prism_control(self) -> PrismControl:
        """Create a prism control.

        Returns
        -------
        PrismControl
            Prism control.

        Examples
        --------
        >>> prism_control = model.control_data.create_prism_control()

        """
        res = _ControlData.create_prism_control(self)
        new_prism_control = PrismControl(self._model, res[0], res[1], res[2])
        self._prism_controls.append(new_prism_control)
        return new_prism_control

    def create_shell_bl_control(self) -> ShellBLControl:
        """Create a ShellBL control.

        Returns
        -------
        ShellBLControl
            ShellBL Control.

        Examples
        --------
        >>> shell_bl_control = model.control_data.create_shell_bl_control()

        """
        res = _ControlData.create_shell_bl_control(self)
        new_shell_bl_control = ShellBLControl(self._model, res[0], res[1], res[2])
        self._shell_bl_controls.append(new_shell_bl_control)
        return new_shell_bl_control

    def create_thin_volume_control(self) -> ThinVolumeControl:
        """Create a thin volume control.

        Returns
        -------
        ThinVolumeControl
            Thin volume control.

        Examples
        --------
        >>> thin_volume_control = model.control_data.create_thin_volume_control()

        """
        res = _ControlData.create_thin_volume_control(self)
        new_thin_volume_control = ThinVolumeControl(self._model, res[0], res[1], res[2])
        self._thin_volume_controls.append(new_thin_volume_control)
        return new_thin_volume_control

    def create_wrapper_control(self) -> WrapperControl:
        """Create a wrapper control with default values.

        Returns
        -------
        WrapperControl
            Wrapper control.

        Notes
        -----
        A wrapper control with default values is created on calling this method.

        Examples
        --------
        >>> wrapper_control = model.control_data.create_wrapper_control()

        """
        res = _ControlData.create_wrapper_control(self)
        new_control = WrapperControl(self._model, res[0], res[1], res[2])
        self._wrapper_controls.append(new_control)
        return new_control

    def create_multi_zone_control(self) -> MultiZoneControl:
        """Create multizone control with defaults.

        Returns
        -------
        multizone
            Returns the multizone control.

        Examples
        --------
        >>> multizone = model.control_data.create_wrapper_control()

        """
        res = _ControlData.create_multi_zone_control(self)
        new_control = MultiZoneControl(self._model, res[0], res[1], res[2])
        self._mz_controls.append(new_control)
        return new_control

    def get_size_control_by_name(self, name: str) -> SizeControl:
        """Get a size control by name.

        Parameters
        ----------
        name : str
            Name of the size control.

        Returns
        -------
        SizeControl
            Size control.

        Examples
        --------
        >>> size_control = model.control_data.get_size_control_by_name("SizeControl-1")

        """
        for size_control in self._size_controls:
            if size_control.name == name:
                return size_control
        return None

    def get_prism_control_by_name(self, name: str) -> PrismControl:
        """Get a prism control by name.

        Parameters
        ----------
        name : str
            Name of the prism control.

        Returns
        -------
        PrismControl
            Prism control.

        Examples
        --------
        >>> prism_control = model.control_data.get_prism_control_by_name("PrismControl-1")

        """
        for prism_control in self._prism_controls:
            if prism_control.name == name:
                return prism_control
        return None

    def get_shell_bl_control_by_name(self, name: str) -> ShellBLControl:
        """Get a shell bl control by name.

        Parameters
        ----------
        name : str
            Name of the shell bl control.

        Returns
        -------
        ShellBLControl
            Shell BL control.

        Examples
        --------
        >>> shell_bl_control = model.control_data.get_shell_bl_control_by_name("ShellBLControl-1")

        """
        for shell_bl_control in self._shell_bl_controls:
            if shell_bl_control.name == name:
                return shell_bl_control
        return None

    def get_thin_volume_control_by_name(self, name: str) -> ThinVolumeControl:
        """Get a thin volume control by name.

        Parameters
        ----------
        name : str
            Name of the thin volume control.

        Returns
        -------
        ThinVolumeControl
            Thin volume control.

        Examples
        --------
        >>> contorl_data = model.control_data
        >>> thin_volume_control = control_data.get_thin_volume_control_by_name(
                                                     "ThinVolumeControl-1")
        """
        for thin_volume_control in self._thin_volume_controls:
            if thin_volume_control.name == name:
                return thin_volume_control
        return None

    def delete_controls(self, control_ids: Iterable[int]) -> DeleteResults:
        """Delete the control for one or more IDs.

        Parameters
        ----------
        control_ids : Iterable[int]
            List of control IDs.

        Returns
        -------
        DeleteResults
            Delete results.

        Examples
        --------
        >>> results = model.control_data.delete_controls([size_control.id, volume_control.id])

        """
        res = _ControlData.delete_controls(self, control_ids=control_ids)
        if res.error_code == ErrorCode.NOERROR:
            for id in control_ids:
                for size_control in self._size_controls:
                    if size_control.id == id:
                        self._size_controls.remove(size_control)
                        break
                for wrapper_control in self._wrapper_controls:
                    if wrapper_control.id == id:
                        self._wrapper_controls.remove(wrapper_control)
                        break
                for prism_control in self._prism_controls:
                    if prism_control.id == id:
                        self._prism_controls.remove(prism_control)
                        break
                for shell_bl_control in self._shell_bl_controls:
                    if shell_bl_control.id == id:
                        self._shell_bl_controls.remove(shell_bl_control)
                        break
                for thin_volume_control in self._thin_volume_controls:
                    if thin_volume_control.id == id:
                        self._thin_volume_controls.remove(thin_volume_control)
                        break
                for volume_control in self._volume_controls:
                    if volume_control.id == id:
                        self._volume_controls.remove(volume_control)
                        break
                for periodic_control in self._periodic_controls:
                    if periodic_control.id == id:
                        self._periodic_controls.remove(periodic_control)
                        break
                for multi_zone_control in self._mz_controls:
                    if multi_zone_control.id == id:
                        self._mz_controls.remove(multi_zone_control)
                        break
        return res

    def _update_size_controls(self, c_data: List):
        self._size_controls = [SizeControl(self._model, c[0], c[1], c[2]) for c in c_data]

    def _update_prism_controls(self, c_data: List):
        self._prism_controls = [PrismControl(self._model, c[0], c[1], c[2]) for c in c_data]

    def _update_shell_bl_controls(self, c_data: List):
        self._shell_bl_controls = [ShellBLControl(self._model, c[0], c[1], c[2]) for c in c_data]

    def _update_wrapper_controls(self, c_data: List):
        self._wrapper_controls = [WrapperControl(self._model, c[0], c[1], c[2]) for c in c_data]

    def _update_multi_zone_controls(self, c_data: List):
        self._mz_controls = [MultiZoneControl(self._model, c[0], c[1], c[2]) for c in c_data]

    def _update_volume_controls(self, c_data: List):
        self._volume_controls = [VolumeControl(self._model, c[0], c[1], c[2]) for c in c_data]

    def _update_periodic_controls(self, c_data: List):
        self._periodic_controls = [PeriodicControl(self._model, c[0], c[1], c[2]) for c in c_data]

    def _update_thin_volume_controls(self, c_data: List):
        self._thin_volume_controls = [
            ThinVolumeControl(self._model, c[0], c[1], c[2]) for c in c_data
        ]

    def create_volume_control(self) -> VolumeControl:
        """Create a volume control.

        Returns
        -------
        VolumeControl
            Volume control.

        Examples
        --------
        >>> volume_control = model.control_data.create_volume_control()

        """
        res = _ControlData.create_volume_control(self)
        new_control = VolumeControl(self._model, res[0], res[1], res[2])
        self._volume_controls.append(new_control)
        return new_control

    def get_volume_control_by_name(self, name: str) -> VolumeControl:
        """Get a volume control by name.

        Parameters
        ----------
        name : str
            Name of the volume control.

        Returns
        -------
        VolumeControl
            Volume control.

        Examples
        --------
        >>> volume_control = model.control_data.get_volume_control_by_name("VolumeControl-1")

        """
        for volume_control in self._volume_controls:
            if volume_control.name == name:
                return volume_control
        return None

    @property
    def size_controls(self) -> List[SizeControl]:
        """Get the size controls.

        Returns
        -------
        List[SizeControl]
            List of size controls.

        Examples
        --------
            >>> size_controls = model.control_data.size_controls
        """
        return self._size_controls

    @property
    def volume_controls(self) -> List[VolumeControl]:
        """Get the volume controls.

        Returns
        -------
        List[VolumeControl]
            List of volume controls.

        Examples
        --------
            >>> volume_controls = model.control_data.volume_controls
        """
        return self._volume_controls

    @property
    def prism_controls(self) -> List[PrismControl]:
        """Get the prism controls.

        Returns
        -------
        List[PrismControl]
            List of prism controls.

        Examples
        --------
        >>> prism_control = model.control_data.prism_controls

        """
        return self._prism_controls

    @property
    def thin_volume_controls(self) -> List[ThinVolumeControl]:
        """Get the thin volume controls.

        Returns
        -------
        List[ThinVolumeControl]
            List of thin volume controls.

        Examples
        --------
        >>> thin_volume_control = model.control_data.thin_volume_controls

        """
        return self._thin_volume_controls

    @property
    def shell_bl_controls(self) -> List[ShellBLControl]:
        """Get the shell bl controls.

        Returns
        -------
        List[ShellBLControl]
            List of shell bl controls.

        Examples
        --------
        >>> shell_bl_control = model.control_data.shell_bl_controls

        """
        return self._shell_bl_controls

    @property
    def wrapper_controls(self) -> List[WrapperControl]:
        """Get the wrapper controls.

        Returns
        -------
        List[WrapperControl]
            List of wrapper controls.

        Examples
        --------
        >>> wrapper_control = model.control_data.wrapper_controls

        """
        return self._wrapper_controls

    @property
    def multi_zone_controls(self) -> List[MultiZoneControl]:
        """Get the multizone controls.

        Returns
        -------
        List[MultiZoneControl]
            Returns the list of multizone controls.

        Examples
        --------
        >>> multi_zone_control = model.control_data.multi_zone_controls

        """
        return self._mz_controls

    def create_periodic_control(self) -> PeriodicControl:
        """Create a periodic control.

        Returns
        -------
        PeriodicControl
            Periodic control.

        Examples
        --------
        >>> periodic_control = model.control_data.create_periodic_control()

        """
        res = _ControlData.create_periodic_control(self)
        new_control = PeriodicControl(self._model, res[0], res[1], res[2])
        self._periodic_controls.append(new_control)
        return new_control

    def get_periodic_control_by_name(self, name: str) -> PeriodicControl:
        """Get a periodic control by name.

        Parameters
        ----------
        name : str
            Name of the periodic control.

        Returns
        -------
        PeriodicControl
            Periodic control.

        Examples
        --------
        >>> periodic_control = model.control_data.get_periodic_control_by_name("PeriodicControl-1")

        """
        for periodic_control in self._periodic_controls:
            if periodic_control.name == name:
                return periodic_control
        return None

    @property
    def periodic_controls(self) -> List[PeriodicControl]:
        """Get the periodic controls.

        Returns
        -------
        List[PeriodicControl]
            List of periodic controls.

        Examples
        --------
            >>> periodic_controls = model.control_data.periodic_controls
        """
        return self._periodic_controls



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Util functions for lucid modules."""
import re


def match_pattern(pattern: str, name: str) -> bool:
    """Pattern matching function for strings.

    Parameters
    ----------
    pattern : str
        Pattern you are looking for in the string.
    name : str
        String where you look for patterns.

    Returns
    -------
    bool
        True if the pattern is found.
    """
    pattern = "^" + pattern.replace("*", ".*").replace("?", ".") + "$"
    x = re.search(pattern, name)
    if x:
        return True
    else:
        return False


def check_name_pattern(name_patterns: str, name: str) -> bool:
    """Check several patterns in one string.

    Parameters
    ----------
    name_patterns : str
        Patterns to check, separated by commas.
        If pattern starts with !, it shouldn't be found.
    name : str
        String where you look for patterns.

    Returns
    -------
    bool
        True if all found.
    """
    patterns = []
    a = name_patterns.split(",")
    for aa in a:
        patterns.append(aa)

    for pattern in patterns:
        bb = pattern.split("!")
        if match_pattern(bb[0].strip(), name):
            if len(bb) > 1:
                nv = False
                for nvbb in bb[1:]:
                    if match_pattern(nvbb.strip(), name):
                        nv = True
                        break
                if not nv:
                    return True
            else:
                return True

    return False



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Module containing WrapperControl related classes and methods."""
from ansys.meshing.prime.autogen.wrappercontrol import WrapperControl as _WrapperControl

# isort: split
from ansys.meshing.prime.autogen.commonstructs import SetNameResults as SetNameResults
from ansys.meshing.prime.internals.comm_manager import CommunicationManager


class WrapperControl(_WrapperControl):
    """Wrapper Control to describe all parameters and controls used for wrapping..

    Parameters
    ----------
    model : Model
        Server model to create WrapperControl object.
    id : int
        Id of the WrapperControl.
    object_id : int
        Object id of the WrapperControl.
    name : str
        Name of the WrapperControl.
    """

    def __init__(self, model: CommunicationManager, id: int, object_id: int, name: str):
        """Initialize the superclass and the ''Model'' variable."""
        self._model = model
        _WrapperControl.__init__(self, model, id, object_id, name)

    def __str__(self) -> str:
        """Get a representation of the class in string format.

        Returns
        -------
        str
            Class data in string format.
        """
        return "Not implemented yet"

    def set_suggested_name(self, name: str) -> SetNameResults:
        """Set the unique name for the wrapper control based on a suggested name.

        Parameters
        ----------
        name : str
            Suggested name for the wrapper control.

        Returns
        -------
        SetNameResults
            Newly assigned name of the wrapper control.


        Examples
        --------
        >>> wrapper_control.set_suggested_name("wrapper_control1")

        """
        result = _WrapperControl.set_suggested_name(self, name)
        self._name = result.assigned_name
        return result

    @property
    def name(self):
        """Get the name of the wrapper control.

        Returns
        -------
        str
            Name of the wrapper control.

        Examples
        --------
        >>> print(wrapper_control.name)

        """
        return self._name



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""This module contains the HidePicked class."""
import os
from typing import TYPE_CHECKING

from ansys.tools.visualization_interface.backends.pyvista.widgets import PlotterWidget
from vtk import vtkPNGReader

if TYPE_CHECKING:
    from ansys.meshing.prime.graphics.plotter import PrimePlotter


class HidePicked(PlotterWidget):
    """Initializes the hide picked button widget.

    This widget lets you hide the picked mesh objects.

    Parameters
    ----------
    prime_plotter : Plotter
        Plotter to apply this widget to.
    """

    def __init__(self, prime_plotter: "PrimePlotter") -> None:
        """Initialize the widget."""
        super().__init__(prime_plotter._backend._pl.scene)
        self.prime_plotter = prime_plotter
        self._picked_dict = self.prime_plotter._backend._picked_dict
        self._object_actors_map = self.prime_plotter._backend._object_to_actors_map
        self._button = self.prime_plotter._backend._pl.scene.add_checkbox_button_widget(
            self.callback,
            position=(5, 660),
            size=30,
            border_size=3,
            color_off="white",
            color_on="white",
        )
        self._removed_actors = []

    def callback(self, state: bool) -> None:
        """Define callback function for the button widget."""
        if state:
            for meshobject in list(self._picked_dict.values()):
                self.prime_plotter._backend.pv_interface.scene.remove_actor(meshobject.actor)
                self._removed_actors.append(meshobject.actor)
        else:
            for actor in self._removed_actors:
                self.prime_plotter._backend._pl.scene.add_actor(actor)

    def update(self) -> None:
        """Define the configuration and representation of the button widget button."""
        vr = self._button.GetRepresentation()
        icon_file = os.path.join(os.path.dirname(__file__), "images", "invert_visibility.png")
        r = vtkPNGReader()
        r.SetFileName(icon_file)
        r.Update()
        image = r.GetOutput()
        vr.SetButtonTexture(0, image)
        vr.SetButtonTexture(1, image)



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Module for communications with the socket server."""
__all__ = ['SocketCommunicator']
import json
import socket
from typing import Optional

import numpy as np

import ansys.meshing.prime.internals.json_utils as json
from ansys.meshing.prime.core.model import Model
from ansys.meshing.prime.internals.communicator import Communicator
from ansys.meshing.prime.internals.error_handling import (
    communicator_error_handler,
    error_code_handler,
)


class NumpyEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, np.ndarray):
            return obj.tolist()
        return super().default(obj)


class SocketCommunicator(Communicator):
    """Manages communication with the socket server.

    Parameters
    ----------
    ip : Optional[str], optional
        IP address where the server is located. The default is ``None``.
    port : Optional[int], optional
        Port where the server is deployed. The default is ``None``.
    timeout : float, optional
        Maximum time to wait for connection. The default is ``10.0``.
    credentials : Any, optional
        Credentials for connecting to the server. The default is ``None``.

    Raises
    ------
    ConnectionError
        Could not connect to server.
    """

    def __init__(
        self,
        ip: Optional[str] = None,
        port: Optional[int] = None,
        timeout: float = 10.0,
        credentials=None,
        **kwargs,
    ):
        """Initialize the server connection."""
        self._models = []
        self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self._socket.bind(('', port))
        self._socket.listen(5)
        self._communicator, self._address = self._socket.accept()

    @error_code_handler
    @communicator_error_handler
    def serve(self, model: Model, command: str, *args, **kwargs) -> dict:
        """Serve model and send a command to the server.

        Parameters
        ----------
        model : Model
            Model to serve.
        command : str
            Command to send.

        Returns
        -------
        dict
            Response from the server.

        Raises
        ------
        RuntimeError
            Can not connect to server.
        """
        self._send_string("ServeJson")
        command = {"Command": command}
        if len(args) > 0:
            command.update({"ObjectID": args[0]})
        if kwargs is not None:
            if "args" in kwargs:
                command.update({"Args": kwargs["args"]})
        self._send_string(json.dumps(command, cls=NumpyEncoder))
        return json.loads(self._recv_string())

    def _send_string(self, data):
        byte_string = (data + '\0').encode('utf-8')
        string_length = len(byte_string).to_bytes(4, 'little')
        self._communicator.send(string_length)
        self._communicator.send(byte_string)

    def _recv_string(self):
        result_len = int.from_bytes(self._communicator.recv(4), byteorder="little")
        result = ""
        if result_len > 0:
            result = self._communicator.recv(result_len).decode("utf-8")
        return result

    def initialize_params(self, model: Model, param_name: str, *args) -> dict:
        """Initialize parameters on the server side.

        Parameters
        ----------
        model : Model
            Model to initialize parameters on.
        param_name : str
            Parameter to initialize.

        Returns
        -------
        dict
            Response from the server.

        Raises
        ------
        RuntimeError
            Can not connect to server.
        """
        self._send_string("GetParamDefaultJSon")
        self._send_string(json.dumps({"ParamName": param_name}))
        return json.loads(self._recv_string())

    def run_on_server(self, model: Model, recipe: str) -> dict:
        """Run commands on the server.

        Run commands on a model on the server.

        Parameters
        ----------
        model : core.Model
            Model to run commands on.
        recipe : str
            Commands to run.

        Returns
        -------
        dict
            Result from the server side.

        Raises
        ------
        RuntimeError
            Bad response from server.
        RuntimeError
            Can not connect to server.
        """
        pass

    def close(self):
        """Close opened channels."""
        self._communicator.close()

    def __del__(self):
        """Close communication when deleting the instance."""
        self.close()

    @property
    def models(self):
        """List of models available."""
        return None



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Process the mesh for visualization in the GUI."""

import enum
from typing import Dict, List, Union

import numpy as np
import pyvista as pv
from ansys.tools.visualization_interface import MeshObjectPlot

import ansys.meshing.prime as prime
from ansys.meshing.prime.autogen.coreobject import CommunicationManager
from ansys.meshing.prime.autogen.meshinfo import MeshInfo
from ansys.meshing.prime.autogen.meshinfostructs import (
    EdgeConnectivityResults,
    FaceAndEdgeConnectivityParams,
    FaceConnectivityResults,
)
from ansys.meshing.prime.core.part import Part
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import CommunicationManager


class ColorByType(enum.IntEnum):
    """Contains the zone types to display."""

    ZONE = 0
    ZONELET = 1
    PART = 2


color_matrix = np.array(
    [
        [155, 186, 126],
        [242, 236, 175],
        [255, 187, 131],
        [194, 187, 97],
        [159, 131, 169],
        [157, 190, 139],
        [233, 218, 158],
        [254, 252, 196],
        [246, 210, 148],
        [215, 208, 198],
        [196, 235, 145],
    ]
)


class DisplayMeshType(enum.IntEnum):
    """Contains the mesh types to display."""

    TOPOFACE = 0
    TOPOEDGE = 1
    FACEZONELET = 2
    EDGEZONELET = 3
    SPLINECONTROLPOINTS = 4
    SPLINESURFACE = 5


class DisplayMeshInfo:
    """Contains the mesh information to display.

    Parameters
    ----------
    id : int, default: 0
        ID of the mesh.
    part_id : int, default: 0
        ID of the part.
    part_name : str, default: None
        Name of the part.
    zone_id : int, default: 0
        ID of the zone.
    zone_name : str, default: None
        Name of the zone.
    display_mesh_type : DisplayMeshType, default: FACEZONELET
        Type of mesh to display.
    """

    def __init__(
        self,
        id=0,
        part_id=0,
        part_name=None,
        zone_id=0,
        zone_name=None,
        display_mesh_type=DisplayMeshType.FACEZONELET,
        has_mesh=False,
    ) -> None:
        """Initialize display mesh information."""
        self.id = id
        self.part_id = part_id
        self.zone_id = zone_id
        self.part_name = part_name
        self.zone_name = zone_name
        self.display_mesh_type = display_mesh_type
        self.has_mesh = has_mesh


def compute_distance(point1, point2) -> float:
    """Compute the distance between two points.

    Parameters
    ----------
    point1 : list
        List with the coordinates of the first point.
    point2 : list
        List with the coordinates of the second point.

    Returns
    -------
    float
        Distance between the two points.
    """
    dist = np.linalg.norm(np.array(point2) - np.array(point1))
    return dist


def compute_face_list_from_structured_nodes(dim):
    """Compute the distances from the nodes.

    Parameters
    ----------
    dim : List[int]
        List with the number of elements in each dimension.

    Returns
    -------
    List
        List with the faces.
    """
    flist = []
    for w in range(dim[2]):
        for u in range(dim[0] - 1):
            for v in range(dim[1] - 1):
                flist.append(4)
                flist.append(u + v * dim[0] + w * dim[0] * dim[1])
                flist.append(u + 1 + v * dim[0] + w * dim[0] * dim[1])
                flist.append(u + 1 + (v + 1) * dim[0] + w * dim[0] * dim[1])
                flist.append(u + (v + 1) * dim[0] + w * dim[0] * dim[1])

    for v in range(dim[1]):
        for u in range(dim[0] - 1):
            for w in range(dim[2] - 1):
                flist.append(4)
                flist.append(u + v * dim[0] + w * dim[0] * dim[1])
                flist.append(u + 1 + v * dim[0] + w * dim[0] * dim[1])
                flist.append(u + 1 + v * dim[0] + (w + 1) * dim[0] * dim[1])
                flist.append(u + v * dim[0] + (w + 1) * dim[0] * dim[1])

    for u in range(dim[0]):
        for v in range(dim[1] - 1):
            for w in range(dim[2] - 1):
                flist.append(4)
                flist.append(u + v * dim[0] + w * dim[0] * dim[1])
                flist.append(u + (v + 1) * dim[0] + w * dim[0] * dim[1])
                flist.append(u + (v + 1) * dim[0] + (w + 1) * dim[0] * dim[1])
                flist.append(u + v * dim[0] + (w + 1) * dim[0] * dim[1])
    return flist


class Mesh(MeshInfo):
    """Processes the mesh for visualization in the GUI.

    Parameters
    ----------
    model : CommunicationManager
        Model to process.
    """

    def __init__(self, model: CommunicationManager):
        """Initialize the mesh object."""
        super().__init__(model)
        self._model = model
        self._unfreeze()
        self._parts_polydata = {}
        self._freeze()

    @property
    def model(self):
        """Model."""
        return self._model

    def get_face_color(self, part: Part, model_type: ColorByType = ColorByType.ZONE):
        """Get the colors of faces.

        Returns
        -------
        List
            List of colors for faces.
        """
        num_colors = int(color_matrix.size / 3)
        if model_type == ColorByType.ZONELET:
            return color_matrix[part.id % num_colors].tolist()
        elif model_type == ColorByType.PART:
            return color_matrix[part.id % num_colors].tolist()
        else:
            return color_matrix[part.id % num_colors].tolist()

    def get_edge_color(self, edge_results: EdgeConnectivityResults, index: int):
        """Get the colors of edges.

        Returns
        -------
        List
            List of colors for edges.
        """
        mesh_type = DisplayMeshType.EDGEZONELET
        if edge_results.topo_edge_ids[index] > 0:
            mesh_type = DisplayMeshType.TOPOEDGE
        num_colors = int(color_matrix.size / 3)
        if mesh_type == DisplayMeshType.EDGEZONELET:
            return color_matrix[index % num_colors].tolist()
        elif mesh_type == DisplayMeshType.TOPOEDGE:
            if edge_results.topo_edge_types[index] == 1:
                return [255, 0, 0]
            elif edge_results.topo_edge_types[index] == 2:
                return [0, 0, 0]
            elif edge_results.topo_edge_types[index] == 3:
                return [0, 255, 255]
            elif edge_results.topo_edge_types[index] == 4:
                return [255, 0, 255]
            elif edge_results.topo_edge_types[index] == 5:
                return [255, 255, 0]
            elif edge_results.topo_edge_types[index] == 6:
                return [128, 0, 128]
            else:
                return color_matrix[edge_results.id % num_colors].tolist()

    def _get_vertices_and_surf_faces(
        self, connectivity_results: FaceConnectivityResults, index
    ) -> Union[np.ndarray, np.ndarray]:
        """Calculate the vertices and faces of the mesh.

        Parameters
        ----------
        connectivity_results : Union[FaceConnectivityResults, EdgeConnectivityResults]
            Results of the connectivity operations.
        index : _type_
            Index of the mesh.

        Returns
        -------
        Union[np.ndarray, np.ndarray]
            Vertices and faces of the mesh.
        """
        node_start = 3 * np.sum(connectivity_results.num_nodes_per_face_zonelet[0:index])
        num_node_coords = 3 * connectivity_results.num_nodes_per_face_zonelet[index]
        face_list_start = np.sum(connectivity_results.num_face_list_per_face_zonelet[0:index])
        num_face_list = connectivity_results.num_face_list_per_face_zonelet[index]
        vertices = connectivity_results.node_coords[
            node_start : node_start + num_node_coords
        ].reshape((-1, 3))
        faces = connectivity_results.face_list[face_list_start : face_list_start + num_face_list]
        return vertices, faces

    def _get_vertices_and_surf_edges(
        self, connectivity_results: EdgeConnectivityResults, index: int
    ) -> Union[np.ndarray, np.ndarray]:
        """Calculate the vertices and faces of the mesh.

        Parameters
        ----------
        connectivity_results : Union[FaceConnectivityResults, EdgeConnectivityResults]
            Results of the connectivity operations.
        index : int
            Index of the mesh.

        Returns
        -------
        Union[np.ndarray, np.ndarray]
            Vertices and faces of the mesh.
        """
        node_start = 3 * np.sum(connectivity_results.num_nodes_per_edge_zonelet[0:index])
        num_node_coords = 3 * connectivity_results.num_nodes_per_edge_zonelet[index]
        edge_list_start = np.sum(connectivity_results.num_edge_list_per_edge_zonelet[0:index])
        num_edge_list = connectivity_results.num_edge_list_per_edge_zonelet[index]
        vertices = connectivity_results.node_coords[
            node_start : node_start + num_node_coords
        ].reshape((-1, 3))
        faces = connectivity_results.edge_list[edge_list_start : edge_list_start + num_edge_list]
        return vertices, faces

    def get_face_polydata(
        self, part_id: int, face_facet_res: FaceConnectivityResults, index: int
    ) -> MeshObjectPlot:
        """Get the polydata object of the faces.

        Parameters
        ----------
        part_id : int
            ID of the part to get the polydata from.
        face_facet_res : FaceConnectivityResults
            Results of the face connectivity.
        index : int
            Index of the face.

        Returns
        -------
        MeshObjectPlot, DisplayMeshInfo
            Mesh to be plotted and information of the mesh to display.
        """
        part = self._model.get_part(part_id)

        vertices, faces = self._get_vertices_and_surf_faces(face_facet_res, index)
        surf = pv.PolyData(vertices, faces)
        fcolor = np.array(self.get_face_color(part, ColorByType.ZONE))
        colors = np.tile(fcolor, (surf.n_faces_strict, 1))
        surf["colors"] = colors
        surf.disp_mesh = self
        has_mesh = True
        if face_facet_res.topo_face_ids[index] > 0:
            display_mesh_type = DisplayMeshType.TOPOFACE
            id = face_facet_res.topo_face_ids[index]
            has_mesh = face_facet_res.mesh_face_ids[index] > 0
        else:
            display_mesh_type = DisplayMeshType.FACEZONELET
            id = face_facet_res.face_zonelet_ids[index]

        if surf.n_points > 0:
            return MeshObjectPlot(part, surf), DisplayMeshInfo(
                id=id,
                part_id=part_id,
                zone_id=face_facet_res.face_zone_ids[index],
                display_mesh_type=display_mesh_type,
                part_name=part.name,
                zone_name=face_facet_res.face_zone_names[index],
                has_mesh=has_mesh,
            )

    def get_edge_polydata(
        self, part_id: int, edge_facet_res: EdgeConnectivityResults, index: int
    ) -> MeshObjectPlot:
        """Get the polydata object of the edges.

        Parameters
        ----------
        part_id : int
            ID of the part to get the polydata from.
        edge_facet_res : EdgeConnectivityResults
            Results of the edge connectivity.
        index : int
            Index of the edge.

        Returns
        -------
        MeshObjectPlot
            Mesh to be displayed.
        """
        part = self._model.get_part(part_id)
        vertices, faces = self._get_vertices_and_surf_edges(edge_facet_res, index)
        edge = pv.PolyData()
        n_edges = edge_facet_res.num_edges_per_edge_zonelet[index]
        edge.points = vertices
        cells = np.full((n_edges, 3), 2, dtype=np.int_)
        i = 0
        j = 0
        while j < len(faces):
            nnodes = faces[j]
            j += 1
            cells[i, 1] = faces[j]
            if nnodes == 2:
                cells[i, 2] = faces[j + 1]
            elif nnodes == 3:
                cells[i, 2] = faces[j + 2]
            j += nnodes
            i += 1
        edge.lines = cells
        ecolor = np.array(self.get_edge_color(edge_facet_res, index))
        colors = np.tile(ecolor, (n_edges, 1))
        edge["colors"] = colors
        edge.disp_mesh = self
        if edge.n_points > 0:
            return MeshObjectPlot(part, edge)

    def get_spline_cp_polydata(self, part_id: int, spline_id: int) -> MeshObjectPlot:
        """Get the polydata object of the spline control points.

        Parameters
        ----------
        part_id : int
            ID of the part to get the polydata from.
        spline_id : int
            ID of the spline.

        Returns
        -------
        MeshObjectPlot
            Mesh to be displayed.
        """
        part = self._model.get_part(part_id)
        spline = part.get_spline(spline_id)
        dim = spline.control_points_count
        vertices = spline.control_points
        faces = self.compute_face_list_from_structured_nodes(dim)
        surf = pv.PolyData(vertices, faces)
        fcolor = np.array([0, 0, 255])
        colors = np.tile(fcolor, (surf.n_faces_strict, 1))
        surf["colors"] = colors
        surf.disp_mesh = self
        if surf.n_points > 0:
            return MeshObjectPlot(part, surf)

    def get_spline_surface_polydata(self, part_id: int, spline_id: int) -> MeshObjectPlot:
        """Get the polydata object of the spline surface.

        Parameters
        ----------
        part_id : int
            ID of the part to get the polydata from.
        spline_id : int
            ID of the spline.

        Returns
        -------
        MeshObjectPlot
            Mesh to be displayed.
        """
        part = self._model.get_part(part_id)
        spline = part.get_spline(spline_id)
        dim = spline.spline_points_count
        vertices = spline.spline_points
        faces = self.compute_face_list_from_structured_nodes(dim)
        surf = pv.PolyData(vertices, faces)
        fcolor = np.array(color_matrix[1])
        colors = np.tile(fcolor, (surf.n_faces_strict, 1))
        surf["colors"] = colors
        surf.disp_mesh = self
        if surf.n_points > 0:
            return MeshObjectPlot(part, surf)

    def get_scoped_polydata(self, scope: "prime.ScopeDefinition", update: bool = False):
        """Get the polydata object of the scoped mesh.

        Parameters
        ----------
        scope : prime.ScopeDefinition
            Scope to get the mesh from.

        Returns
        -------
        pv.PolyData
            PyVista mesh object.
        """
        self.as_polydata(update=update)
        parts = self._model.control_data.get_scope_parts(scope)

        # Update the polydata if any part is not in the dictionary
        if len(set(parts).intersection(set(self._parts_polydata.keys()))) != len(parts):
            self.update_pd(parts)
        scoped_pd = {}
        scope_def = scope
        for part_id in parts:
            part = self._model.get_part(part_id)
            scope_def.part_expression = part.name
            disp_data = None
            disp_ids = []
            if scope.entity_type == prime.ScopeEntity.FACEZONELETS:
                disp_ids = self._model.control_data.get_scope_face_zonelets(
                    scope=scope_def, params=prime.ScopeZoneletParams(model=self._model)
                )
                disp_data = self._parts_polydata[part_id]["faces"]

            if disp_data is not None:
                temp_pd = []
                for disp_mesh in disp_data:
                    if disp_mesh[1].id in disp_ids:
                        temp_pd.append(disp_mesh)
                temp_key = {}
                if len(temp_pd) > 0:
                    temp_key["faces"] = temp_pd
                    scoped_pd[part_id] = temp_key

        # in case the scoped_pd is empty, the mesh must be reinitialized
        # to get the updated changes from the backend
        if len(scoped_pd) == 0:
            self.__init__(self._model)
            return self.get_scoped_polydata(scope, update=True)
        return scoped_pd

    def update_pd(self, part_ids) -> Dict[int, Dict[str, list[(pv.PolyData, Part)]]]:
        """Update the polydata object of the mesh.

        Parameters
        ----------
        part_ids : List[int]
            List of part IDs to update.

        Returns
        -------
        Dict[int, Dict[str, List[(pv.PolyData, Part)]]
            Dictionary with the polydata objects.
        """
        with prime.numpy_array_optimization_enabled():
            facet_result = self.get_face_and_edge_connectivity(
                part_ids, FaceAndEdgeConnectivityParams(model=self._model)
            )
        self._parts_polydata = {}
        for i, part_id in enumerate(facet_result.part_ids):
            part = self._model.get_part(part_id)
            splines = part.get_splines()
            part_polydata = {}
            face_polydata_list = [
                self.get_face_polydata(part_id, face_fet_result, j)
                for face_fet_result in facet_result.face_connectivity_result_per_part
                for j in range(0, len(face_fet_result.face_zonelet_ids))
            ]

            edge_polydata_list = [
                self.get_edge_polydata(part_id, edge_facet_result, j)
                for edge_facet_result in facet_result.edge_connectivity_result_per_part
                for j in range(0, len(edge_facet_result.edge_zonelet_ids))
            ]

            spline_cp_polydata_list = [self.get_spline_cp_polydata(part_ids[i], j) for j in splines]

            spline_surface_polydata_list = [
                self.get_spline_surface_polydata(part_ids[i], j) for j in splines
            ]

            part_polydata["faces"] = face_polydata_list
            part_polydata["edges"] = edge_polydata_list
            part_polydata["ctrlpts"] = spline_cp_polydata_list
            part_polydata["splinesurf"] = spline_surface_polydata_list
            self._parts_polydata[part_id] = part_polydata
        return self._parts_polydata

    def as_polydata(
        self,
        update: bool = False,
    ) -> Dict[int, Dict[str, List[tuple[pv.PolyData, Part]]]]:
        """Return the mesh as a ``pv.PolyData`` object.

        Parameters
        ----------
        update : bool, default: False
            Update the polydata.

        Returns
        -------
        Dict[int, Dict[str, List[(pv.PolyData, Part)]]
            Dictionary with the polydata objects.
        """
        if not self._parts_polydata or update:
            part_ids = [part.id for part in self._model.parts]
            self.update_pd(part_ids)
        return self._parts_polydata

    @property
    def id(self):
        """Return the ID of the mesh.

        Returns
        -------
        int
            ID of the mesh.
        """
        return self._id

    @property
    def part_id(self):
        """Return the part ID of the mesh.

        Returns
        -------
        int
            Part ID of the mesh.
        """
        return self._part_id

    @property
    def zone_id(self):
        """Return the zone ID of the mesh.

        Returns
        -------
        int
            Zone ID of the mesh.
        """
        return self._zone_id



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Module for client communication implementations."""

import logging
import os

import ansys.meshing.prime.examples as examples
import ansys.meshing.prime.internals.config as config
import ansys.meshing.prime.internals.defaults as defaults
import ansys.meshing.prime.internals.utils as utils
from ansys.meshing.prime.core.model import Model
from ansys.meshing.prime.internals.utils import terminate_process

__all__ = ['Client']


class Client(object):
    """Provides the ``Client`` class for PyPrimeMesh.

    Parameters
    ----------
    server_process : Any, optional
        Server process in the system. The default is ``None``.
    ip : str, optional
        IP address where the server is located. The default is ``defaults.ip()``.
    port : int, optional
        Port where the server is deployed. The default is ``defaults.port()``.
    timeout : float, optional
        Maximum time to wait for connection. The default is ``defaults.connection_timeout()``.
    credentials : Any, optional
        Credentials to connect to the server. The default is ``None``.

    Raises
    ------
    ValueError
        Failed to load the communicator.
    """

    def __init__(
        self,
        *,
        server_process=None,
        ip: str = defaults.ip(),
        port: int = defaults.port(),
        timeout: float = defaults.connection_timeout(),
        credentials=None,
        communicator_type="grpc",
        **kwargs,
    ):
        """Initialize the client."""
        self._default_model: Model = None
        local = kwargs.get('local', False)
        if local and server_process is not None:
            raise ValueError('Local client cannot be instantiated with a server process')
        self._local = local
        self._process = server_process
        self._comm = None
        if not local:
            if communicator_type == "grpc":
                try:
                    from ansys.meshing.prime.internals.grpc_communicator import (
                        GRPCCommunicator,
                    )

                    channel = kwargs.get('channel', None)
                    if channel is not None:
                        self._comm = GRPCCommunicator(channel=channel, timeout=timeout)
                    else:
                        self._comm = GRPCCommunicator(
                            ip=ip, port=port, timeout=timeout, credentials=credentials
                        )
                        setattr(self, 'port', port)
                except ImportError as err:
                    logging.getLogger('PyPrimeMesh').error(
                        f'Failed to load grpc_communicator with message: {err.msg}'
                    )
                    raise
                except ConnectionError:
                    self.exit()

                    logging.getLogger('PyPrimeMesh').error('Failed to connect to PRIME GRPC server')
                    raise
            elif communicator_type == "socket":
                from ansys.meshing.prime.internals.socket_communicator import (
                    SocketCommunicator,
                )

                self._comm = SocketCommunicator(ip=ip, port=port)
                setattr(self, 'port', port)
            else:
                logging.getLogger('PyPrimeMesh').error(f'Invalid server type: {communicator_type}')
                raise

        else:
            try:
                from ansys.meshing.prime.internals.prime_communicator import (
                    PrimeCommunicator,
                )

                self._comm = PrimeCommunicator()
            except ImportError as err:
                logging.getLogger('PyPrimeMesh').error(
                    f'Failed to load prime_communicator with message: {err.msg}'
                )

    @property
    def model(self):
        """Get model associated with the client."""
        if self._default_model is None and hasattr(self._comm, 'models'):
            if self._comm.models:
                model_info = self._comm.models[0]
                self._default_model = Model(
                    self._comm, model_info['id'], model_info['index'], "Default"
                )

        if self._default_model is None:
            # This assumes that the Model is always object id 1....
            self._default_model = Model(self._comm, 1, 1, "Default")
        return self._default_model

    def run_on_server(self, recipe: str):
        """Run a recipe on the server.

        Parameters
        ----------
        recipe: str
            Recipe to run. This must be a valid Python script.
        """
        if self._comm is not None:
            result = self._comm.run_on_server(recipe)
            return result['Results']

    def exit(self):
        """Close the connection with the server.

        If the client has launched the server, this method also
        kills the server process.

        Examples
        --------
        >>> import ansys.meshing.prime as prime
        >>> prime_client = prime.launch_prime() # This launches a server process.
        >>> model = prime_client.model
        >>> fileio = prime.FileIO(model)
        >>> result = fileio.read_pmdat('example.pmdat', prime.FileReadParams(model=model))
        >>> print(result)
        >>> prime_client.exit() # Sever connection with server and kill the server.
        """
        if self._comm is not None:
            self._comm.close()
            self._comm = None
        if self._process is not None:
            assert self._local == False
            terminate_process(self._process)
            self._process = None

        if config.using_container():
            container_name = getattr(self, 'container_name')
            utils.stop_prime_github_container(container_name)
        elif config.has_pim():
            self.remote_instance.delete()
            self.pim_client.close()

        clear_examples = bool(int(os.environ.get('PYPRIMEMESH_CLEAR_EXAMPLES', '1')))
        if clear_examples:
            download_manager = examples.DownloadManager()
            download_manager.clear_download_cache()

    def __enter__(self):
        """Open client."""
        return self

    def __exit__(self, type, value, traceback):
        """Close communication with the server when deleting the instance."""
        self.exit()



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Module for ColorByTypeWidget."""
import enum
import os
from typing import TYPE_CHECKING

import numpy as np
from ansys.tools.visualization_interface.backends.pyvista.widgets import PlotterWidget
from vtk import vtkPNGReader

if TYPE_CHECKING:
    from ansys.meshing.prime.graphics.plotter import PrimePlotter

from ansys.meshing.prime.core.mesh import DisplayMeshInfo

color_matrix = np.array(
    [
        [155, 186, 126],
        [242, 236, 175],
        [255, 187, 131],
        [194, 187, 97],
        [159, 131, 169],
        [157, 190, 139],
        [233, 218, 158],
        [254, 252, 196],
        [246, 210, 148],
        [215, 208, 198],
        [196, 235, 145],
    ]
)


class ColorByType(enum.IntEnum):
    """Contains the zone types to display."""

    ZONE = 0
    ZONELET = 1
    PART = 2


class ColorByTypeWidget(PlotterWidget):
    """Initializes the color by the type button widget.

    This widget lets you change the color of the mesh
    based on the zone, zonelet, or part.

    Parameters
    ----------
    prime_plotter : Plotter
        Plotter object to use.
    """

    def __init__(self, prime_plotter: "PrimePlotter") -> None:
        """Initialize the widget."""
        super().__init__(prime_plotter._backend._pl.scene)
        self.prime_plotter = prime_plotter
        self._object_actors_map = self.prime_plotter._backend.pv_interface._object_to_actors_map
        self._info_actor_map = self.prime_plotter._info_actor_map
        self._button = self.prime_plotter._backend.pv_interface.scene.add_checkbox_button_widget(
            self.callback, position=(5, 630), size=30, border_size=3
        )
        self._button.GetRepresentation().SetNumberOfStates(3)
        self._color_type = ColorByType.ZONE

    def callback(self, state) -> None:
        """Define the callback function for the button widget."""
        color_type = ColorByType(self._button.GetRepresentation().GetState())
        for actor, object in self._object_actors_map.items():
            if actor in self.prime_plotter._info_actor_map:
                mesh_info = self.prime_plotter._info_actor_map[actor]
                actor.prop.color = self.set_color_by_type(color_type, mesh_info)
                self.update(color_type)

    def update(self, color_type=ColorByType.ZONE) -> None:
        """Define the configuration and representation of the button widget button.

        Parameters
        ----------
        color_type : ColorByType, default: ColorByType.ZONE
            Color type to use.
        """
        vr = self._button.GetRepresentation()
        icon_file = os.path.join(os.path.dirname(__file__), "images", "bin.png")

        if color_type == ColorByType.ZONE:
            icon_file = os.path.join(os.path.dirname(__file__), "images", "bin.png")
        elif color_type == ColorByType.ZONELET:
            icon_file = os.path.join(os.path.dirname(__file__), "images", "surface_body.png")
        elif color_type == ColorByType.PART:
            icon_file = os.path.join(os.path.dirname(__file__), "images", "parts.png")
        reader = vtkPNGReader()
        reader.SetFileName(icon_file)
        reader.Update()
        image = reader.GetOutput()
        vr.SetButtonTexture(0, image)
        vr.SetButtonTexture(1, image)
        vr.SetButtonTexture(2, image)

    def set_color_by_type(self, color_type: ColorByType, mesh_info: DisplayMeshInfo):
        """Get the colors of faces.

        Parameters
        ----------
        color_type : ColorByType
            Color type to use.
        mesh_info : DisplayMeshInfo
            Mesh information that generates an appropriate color.

        Returns
        -------
        List
            List of colors for faces.
        """
        num_colors = int(color_matrix.size / 3)
        if color_type == ColorByType.ZONELET:
            return color_matrix[mesh_info.id % num_colors].tolist()
        elif color_type == ColorByType.PART:
            return color_matrix[mesh_info.part_id % num_colors].tolist()
        else:
            return color_matrix[mesh_info.zone_id % num_colors].tolist()



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""" Auto-generated file. DO NOT MODIFY """
from __future__ import annotations
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.params.primestructs import *
from ansys.meshing.prime.autogen.coreobject import *
from typing import List, Any, Union

class SurfaceSearch(CoreObject):
    """SurfaceSearch allows you to check surface mesh quality.

    SurfaceSearch performs surface mesh quality check based on different face quality measures.


    Parameters
    ----------
    model : Model
        Server model to create SurfaceSearch object.
    """

    def __init__(self, model: CommunicationManager):
        """ Initialize SurfaceSearch """
        self._model = model
        self._comm = model._communicator
        command_name = "PrimeMesh::SurfaceSearch/Construct"
        args = {"ModelID" : model._object_id , "MaxID" : -1 }
        result = self._comm.serve(model, command_name, args=args)
        self._object_id = result["ObjectIndex"]
        self._freeze()

    def __enter__(self):
        """ Enter context for SurfaceSearch. """
        return self

    def __exit__(self, type, value, traceback) :
        """ Exit context for SurfaceSearch. """
        command_name = "PrimeMesh::SurfaceSearch/Destruct"
        self._comm.serve(self._model, command_name, self._object_id, args={})

    def search_zonelets_by_quality(self, part_id : int, face_zonelets : Iterable[int], register_id : int, params : SearchByQualityParams) -> SearchByQualityResults:
        """ Search face zonelets with the provided quality parameters and applies register id on face elements found.


        Parameters
        ----------
        part_id : int
            Id of the part.
        face_zonelets : Iterable[int]
            Ids of the face zonelets whose element search is performed.
        register_id : int
            Id used to register face elements found by the search.
        params : SearchByQualityParams
            Quality parameters used to search face elements.

        Returns
        -------
        SearchByQualityResults
            Returns the SearchByQualityResults.

        Examples
        --------
        >>> surf_search = prime.SurfaceSearch(model=model)
        >>> results = surf_search.search_zonelets_by_quality(part_id, face_zonelets, register_id, prime.SearchByQualityParams(model=model))

        """
        if not isinstance(part_id, int):
            raise TypeError("Invalid argument type passed for 'part_id'. Valid argument type is int.")
        if not isinstance(face_zonelets, Iterable):
            raise TypeError("Invalid argument type passed for 'face_zonelets'. Valid argument type is Iterable[int].")
        if not isinstance(register_id, int):
            raise TypeError("Invalid argument type passed for 'register_id'. Valid argument type is int.")
        if not isinstance(params, SearchByQualityParams):
            raise TypeError("Invalid argument type passed for 'params'. Valid argument type is SearchByQualityParams.")
        args = {"part_id" : part_id,
        "face_zonelets" : face_zonelets,
        "register_id" : register_id,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::SurfaceSearch/SearchZoneletsByQuality"
        self._model._print_logs_before_command("search_zonelets_by_quality", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("search_zonelets_by_quality", SearchByQualityResults(model = self._model, json_data = result))
        return SearchByQualityResults(model = self._model, json_data = result)

    def search_zonelets_by_self_intersections(self, part_id : int, face_zonelets : Iterable[int], register_id : int, params : SearchBySelfIntersectionParams) -> SearchByIntersectionResults:
        """ Search face zonelets to identify face elements intersecting with each other.


        Parameters
        ----------
        part_id : int
            Id of part.
        face_zonelets : Iterable[int]
            Ids of face zonelets.
        register_id : int
            Id of register.
        params : SearchBySelfIntersectionParams
            Parameters used to identify face elements by self intersection.

        Returns
        -------
        SearchByIntersectionResults
            Returns the SearchByIntersectionResults.

        Examples
        --------
        >>> surf_search = SurfaceSearch(model=model)
        >>> results = surf_search.search_zonelets_by_self_intersections(part_id, face_zonelets, register_id, params)

        """
        if not isinstance(part_id, int):
            raise TypeError("Invalid argument type passed for 'part_id'. Valid argument type is int.")
        if not isinstance(face_zonelets, Iterable):
            raise TypeError("Invalid argument type passed for 'face_zonelets'. Valid argument type is Iterable[int].")
        if not isinstance(register_id, int):
            raise TypeError("Invalid argument type passed for 'register_id'. Valid argument type is int.")
        if not isinstance(params, SearchBySelfIntersectionParams):
            raise TypeError("Invalid argument type passed for 'params'. Valid argument type is SearchBySelfIntersectionParams.")
        args = {"part_id" : part_id,
        "face_zonelets" : face_zonelets,
        "register_id" : register_id,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::SurfaceSearch/SearchZoneletsBySelfIntersections"
        self._model._print_logs_before_command("search_zonelets_by_self_intersections", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("search_zonelets_by_self_intersections", SearchByIntersectionResults(model = self._model, json_data = result))
        return SearchByIntersectionResults(model = self._model, json_data = result)

    def search_zonelets_by_spikes(self, part_id : int, face_zonelets : Iterable[int], register_id : int, params : SearchBySpikeParams) -> SearchBySpikeResults:
        """ Search face zonelets to identify spikes.

        Search face zonelets to identify spikes with provided spike parameters

        Parameters
        ----------
        part_id : int
            Id of part.
        face_zonelets : Iterable[int]
            Ids of face zonelets.
        register_id : int
            Id of register.
        params : SearchBySpikeParams
            Parameters used to identify spikes.

        Returns
        -------
        SearchBySpikeResults
            Returns the SearchBySpikeResults.

        Examples
        --------
        >>> surf_search = SurfaceSearch(model=model)
        >>> results = surf_search.search_zonelets_by_spikes(part_id, face_zonelets, register_id, params)

        """
        if not isinstance(part_id, int):
            raise TypeError("Invalid argument type passed for 'part_id'. Valid argument type is int.")
        if not isinstance(face_zonelets, Iterable):
            raise TypeError("Invalid argument type passed for 'face_zonelets'. Valid argument type is Iterable[int].")
        if not isinstance(register_id, int):
            raise TypeError("Invalid argument type passed for 'register_id'. Valid argument type is int.")
        if not isinstance(params, SearchBySpikeParams):
            raise TypeError("Invalid argument type passed for 'params'. Valid argument type is SearchBySpikeParams.")
        args = {"part_id" : part_id,
        "face_zonelets" : face_zonelets,
        "register_id" : register_id,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::SurfaceSearch/SearchZoneletsBySpikes"
        self._model._print_logs_before_command("search_zonelets_by_spikes", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("search_zonelets_by_spikes", SearchBySpikeResults(model = self._model, json_data = result))
        return SearchBySpikeResults(model = self._model, json_data = result)

    def search_zonelets_by_folds(self, part_id : int, face_zonelets : Iterable[int], register_id : int, params : SearchByFoldsParams) -> SearchByFoldsResults:
        """ Search face zonelets to identify folds.

        Search face zonelets to identify folds with provided folds parameters.

        Parameters
        ----------
        part_id : int
            Id of part.
        face_zonelets : Iterable[int]
            Ids of face zonelets.
        register_id : int
            Id of register.
        params : SearchByFoldsParams
            Parameters used to identify folds.

        Returns
        -------
        SearchByFoldsResults
            Returns the SearchByFoldsResults.

        Examples
        --------
        >>> surf_search = SurfaceSearch(model=model)
        >>> results = surf_search.search_zonelets_by_folds(part_id, face_zonelets, register_id, params)

        """
        if not isinstance(part_id, int):
            raise TypeError("Invalid argument type passed for 'part_id'. Valid argument type is int.")
        if not isinstance(face_zonelets, Iterable):
            raise TypeError("Invalid argument type passed for 'face_zonelets'. Valid argument type is Iterable[int].")
        if not isinstance(register_id, int):
            raise TypeError("Invalid argument type passed for 'register_id'. Valid argument type is int.")
        if not isinstance(params, SearchByFoldsParams):
            raise TypeError("Invalid argument type passed for 'params'. Valid argument type is SearchByFoldsParams.")
        args = {"part_id" : part_id,
        "face_zonelets" : face_zonelets,
        "register_id" : register_id,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::SurfaceSearch/SearchZoneletsByFolds"
        self._model._print_logs_before_command("search_zonelets_by_folds", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("search_zonelets_by_folds", SearchByFoldsResults(model = self._model, json_data = result))
        return SearchByFoldsResults(model = self._model, json_data = result)

    def search_zonelets_by_invalid_normals(self, part_id : int, face_zonelets : Iterable[int], register_id : int) -> SearchByInvalidNormalsResults:
        """ Search face zonelets to identify faces with invalid normals.

        Search face zonelets to identify faces with invalid normals.

        Parameters
        ----------
        part_id : int
            Id of part.
        face_zonelets : Iterable[int]
            Ids of face zonelets.
        register_id : int
            Id of register.

        Returns
        -------
        SearchByInvalidNormalsResults
            Returns the SearchByInvalidNormalsResults.

        Examples
        --------
        >>> surf_search = SurfaceSearch(model=model)
        >>> results = surf_search.search_zonelets_by_invalid_normals(part_id, face_zonelets, register_id)

        """
        if not isinstance(part_id, int):
            raise TypeError("Invalid argument type passed for 'part_id'. Valid argument type is int.")
        if not isinstance(face_zonelets, Iterable):
            raise TypeError("Invalid argument type passed for 'face_zonelets'. Valid argument type is Iterable[int].")
        if not isinstance(register_id, int):
            raise TypeError("Invalid argument type passed for 'register_id'. Valid argument type is int.")
        args = {"part_id" : part_id,
        "face_zonelets" : face_zonelets,
        "register_id" : register_id}
        command_name = "PrimeMesh::SurfaceSearch/SearchZoneletsByInvalidNormals"
        self._model._print_logs_before_command("search_zonelets_by_invalid_normals", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("search_zonelets_by_invalid_normals", SearchByInvalidNormalsResults(model = self._model, json_data = result))
        return SearchByInvalidNormalsResults(model = self._model, json_data = result)

    def search_zonelets_by_thin_strips(self, part_id : int, face_zonelets : Iterable[int], register_id : int, params : SearchByThinStripParams) -> SearchByThinStripResults:
        """ Search face zonelets to identify face element of thin strips(single layer of triangles between features).


        Parameters
        ----------
        part_id : int
            Id of part.
        face_zonelets : Iterable[int]
            Ids of face zonelets.
        register_id : int
            Id of register.
        params : SearchByThinStripParams
            Parameters used to identify face elements of thin strips.

        Returns
        -------
        SearchByThinStripResults
            Returns the SearchByThinStripResults.

        Examples
        --------
        >>> surf_search = SurfaceSearch(model=model)
        >>> results = surf_search.search_zonelets_by_thin_strips(part_id, face_zonelets, register_id, params)

        """
        if not isinstance(part_id, int):
            raise TypeError("Invalid argument type passed for 'part_id'. Valid argument type is int.")
        if not isinstance(face_zonelets, Iterable):
            raise TypeError("Invalid argument type passed for 'face_zonelets'. Valid argument type is Iterable[int].")
        if not isinstance(register_id, int):
            raise TypeError("Invalid argument type passed for 'register_id'. Valid argument type is int.")
        if not isinstance(params, SearchByThinStripParams):
            raise TypeError("Invalid argument type passed for 'params'. Valid argument type is SearchByThinStripParams.")
        args = {"part_id" : part_id,
        "face_zonelets" : face_zonelets,
        "register_id" : register_id,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::SurfaceSearch/SearchZoneletsByThinStrips"
        self._model._print_logs_before_command("search_zonelets_by_thin_strips", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("search_zonelets_by_thin_strips", SearchByThinStripResults(model = self._model, json_data = result))
        return SearchByThinStripResults(model = self._model, json_data = result)

    def get_surface_quality_summary(self, params : SurfaceQualitySummaryParams) -> SurfaceQualitySummaryResults:
        """ Gets the surface quality summary.

        Diagnose surface quality for the given scope and face quality measures provided by the surface quality summary parameters.
        Uses default quality limit if not specified with params.

        Parameters
        ----------
        params : SurfaceQualitySummaryParams
            Surface quality summary parameters.

        Returns
        -------
        SurfaceQualitySummaryResults
            Returns the SurfaceQualitySummaryResults.

        Examples
        --------
        >>> surf_search = SurfaceSearch(model=model)
        >>> results = surf_search.get_surface_quality_summary(SurfaceQualitySummaryParams(model=model))

        """
        if not isinstance(params, SurfaceQualitySummaryParams):
            raise TypeError("Invalid argument type passed for 'params'. Valid argument type is SurfaceQualitySummaryParams.")
        args = {"params" : params._jsonify()}
        command_name = "PrimeMesh::SurfaceSearch/GetSurfaceQualitySummary"
        self._model._print_logs_before_command("get_surface_quality_summary", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_surface_quality_summary", SurfaceQualitySummaryResults(model = self._model, json_data = result))
        return SurfaceQualitySummaryResults(model = self._model, json_data = result)

    def get_surface_diagnostic_summary(self, params : SurfaceDiagnosticSummaryParams) -> SurfaceDiagnosticSummaryResults:
        """ Gets the surface diagnostic summary.

        Diagnose surface connectivity for the given scope and controls provided by the surface diagnostic summary parameters.

        Parameters
        ----------
        params : SurfaceDiagnosticSummaryParams
            Surface diagnostic summary parameters.

        Returns
        -------
        SurfaceDiagnosticSummaryResults
            Returns the SurfaceDiagnosticSummaryResults.

        Examples
        --------
        >>> surf_search = SurfaceSearch(model=model)
        >>> results = surf_search.get_surface_diagnostics_summary(SurfaceDiagnosticSummaryParams(model=model))

        """
        if not isinstance(params, SurfaceDiagnosticSummaryParams):
            raise TypeError("Invalid argument type passed for 'params'. Valid argument type is SurfaceDiagnosticSummaryParams.")
        args = {"params" : params._jsonify()}
        command_name = "PrimeMesh::SurfaceSearch/GetSurfaceDiagnosticSummary"
        self._model._print_logs_before_command("get_surface_diagnostic_summary", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_surface_diagnostic_summary", SurfaceDiagnosticSummaryResults(model = self._model, json_data = result))
        return SurfaceDiagnosticSummaryResults(model = self._model, json_data = result)

    def get_search_info_by_register_id(self, face_zonelets : Iterable[int], register_id : int, params : SearchInfoByRegisterIdParams) -> SearchInfoByRegisterIdResults:
        """ Gets search information regarding registered face elements of provided zonelets using a register id.


        Parameters
        ----------
        face_zonelets : Iterable[int]
            Ids of the face zonelets to search in.
        register_id : int
            An integer register id.
        params : SearchInfoByRegisterIdParams
            Parameters for retrieveing information on registered faces.

        Returns
        -------
        SearchInfoByRegisterIdResults
            Returns the SearchInfoByRegisterIdResults.

        Examples
        --------
        >>> surf_search = SurfaceSearch(model=model)
        >>> params = prime.SearchInfoByRegisterIdParams(model=model)
        >>> results = surf_search.get_search_info_by_register_id(face_zonelets, register_id, params)

        """
        if not isinstance(face_zonelets, Iterable):
            raise TypeError("Invalid argument type passed for 'face_zonelets'. Valid argument type is Iterable[int].")
        if not isinstance(register_id, int):
            raise TypeError("Invalid argument type passed for 'register_id'. Valid argument type is int.")
        if not isinstance(params, SearchInfoByRegisterIdParams):
            raise TypeError("Invalid argument type passed for 'params'. Valid argument type is SearchInfoByRegisterIdParams.")
        args = {"face_zonelets" : face_zonelets,
        "register_id" : register_id,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::SurfaceSearch/GetSearchInfoByRegisterId"
        self._model._print_logs_before_command("get_search_info_by_register_id", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("get_search_info_by_register_id", SearchInfoByRegisterIdResults(model = self._model, json_data = result))
        return SearchInfoByRegisterIdResults(model = self._model, json_data = result)

    def check_face_deviation(self, source_face_zonelets : Iterable[int], target_face_zonelets : Iterable[int], params : CheckFaceDeviationParams) -> CheckFaceDeviationResults:
        """ Gets information regarding the number of faces with a deviation higher than the tolerance.


        Parameters
        ----------
        source_face_zonelets : Iterable[int]
            Scope of reference zonelets from which the deviation is checked.
        target_face_zonelets : Iterable[int]
            Scope of target zonelets for which the deviation is checked.
        params : CheckFaceDeviationParams
            Parameters for retrieving information while performing check face deviation operation.

        Returns
        -------
        CheckFaceDeviationResults
            Returns the CheckFaceDeviationResults.

        Examples
        --------
        >>> surf_search = SurfaceSearch(model=model)
        >>> params = prime.CheckFaceDeviationParams()
        >>> results = surf_search.check_face_deviation(source_scope, reference_scope, params)

        """
        if not isinstance(source_face_zonelets, Iterable):
            raise TypeError("Invalid argument type passed for 'source_face_zonelets'. Valid argument type is Iterable[int].")
        if not isinstance(target_face_zonelets, Iterable):
            raise TypeError("Invalid argument type passed for 'target_face_zonelets'. Valid argument type is Iterable[int].")
        if not isinstance(params, CheckFaceDeviationParams):
            raise TypeError("Invalid argument type passed for 'params'. Valid argument type is CheckFaceDeviationParams.")
        args = {"source_face_zonelets" : source_face_zonelets,
        "target_face_zonelets" : target_face_zonelets,
        "params" : params._jsonify()}
        command_name = "PrimeMesh::SurfaceSearch/CheckFaceDeviation"
        self._model._print_logs_before_command("check_face_deviation", args)
        result = self._comm.serve(self._model, command_name, self._object_id, args=args)
        self._model._print_logs_after_command("check_face_deviation", CheckFaceDeviationResults(model = self._model, json_data = result))
        return CheckFaceDeviationResults(model = self._model, json_data = result)



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Module for abstract communicator."""
from abc import abstractmethod


class Communicator(object):
    """Provides the abstract class for the server communicator."""

    @abstractmethod
    def serve(self, model, command, *args, **kwargs) -> dict:
        """Serve the model and send a command to the server.

        Parameters
        ----------
        model : Model
            Model to serve.
        command : str
            Command to send to the server.

        Returns
        -------
        dict
            Response from the server.
        """
        pass

    @abstractmethod
    def initialize_params(self, model, param_name: str, *args) -> dict:
        """Initialize parameters on the server side.

        Parameters
        ----------
        model : Model
            Model to initialize parameter on.
        param_name : str
            Parameter to initialize.

        Returns
        -------
        dict
            Response from the server.
        """
        pass

    @abstractmethod
    def run_on_server(self, model, recipe: str) -> dict:
        """Run the command on the server side.

        Parameters
        ----------
        model : Model
            Model to run commands on.
        recipe : str
            Command to run.

        Returns
        -------
        dict
            Response from the server.
        """
        pass

    @abstractmethod
    def import_cad(self, model, file_name: str, *args) -> dict:
        """Import a CAD file from local.

        Parameters
        ----------
        model : Model
            Model to import.
        file_name : str
            Name of the CAD file.

        Returns
        -------
        dict
            Response from the server.
        """
        pass

    @abstractmethod
    def close(self):
        """Close communication when deleting the instance."""
        pass



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Module for surfer data."""
from typing import Iterable

# isort: split
from ansys.meshing.prime.autogen.surfer import Surfer as _Surfer

# isort: split
from ansys.meshing.prime.autogen.coreobject import CoreObject
from ansys.meshing.prime.autogen.surferstructs import (
    CreateShellBLResults,
    LocalSurferParams,
    LocalSurferResults,
    ShellBLParams,
    SurferParams,
    SurferResults,
)
from ansys.meshing.prime.core.model import Model


class Surfer(CoreObject):
    """Generates surface mesh.

    Performs surface meshing using various surface meshing algorithms on topofaces or face zonelets.
    For example, constant size or volumetric size surface meshing.

    Parameters
    ----------
    model : Model
        Server model to create Surfer object.
    """

    __doc__ = _Surfer.__doc__

    def __init__(self, model: Model):
        """Initialize a surfer instance."""
        self._model = model

    def mesh_topo_faces(
        self, part_id: int, topo_faces: Iterable[int], params: SurferParams
    ) -> SurferResults:
        """Perform meshing on TopoFaces with given parameters.

        Parameters
        ----------
        part_id : int
            ID of part.
        topo_faces : Iterable[int]
            IDs of the TopoFaces.
        params : SurferParams
            Surfer parameters.

        Returns
        -------
        SurferResults
            Results from performing meshing on the TopoFaces.

        Examples
        --------
        >>> surfer = prime.Surfer(model)
        >>> surfer_params = prime.SurferParams(model)
        >>> results = surfer.mesh_topo_faces(part_id, topo_faces, surfer_params)

        """
        with _Surfer(model=self._model, part_id=part_id) as surfer:
            return surfer.mesh_topo_faces(topo_faces, params)

    def create_shell_bl_using_controls(
        self, part_id: int, shell_bl_control_ids: Iterable[int], shell_bl_params: ShellBLParams
    ) -> CreateShellBLResults:
        """Create ShellBL using data stored in controls.

        Parameters
        ----------
        part_id : int
            Id of the part.
        shell_bl_control_ids : Iterable[int]
            Ids of ShellBL control.
        shell_bl_params : ShellBLParams
            Parameters related to ShellBL.

        Returns
        -------
        CreateShellBLResults
            Returns the CreateShellBLResults.


        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.

        Examples
        --------
        >>> results = surfer.create_shell_bl_using_controls(part_id,
                                                            shell_bl_control_ids,
                                                            shell_bl_params)

        """
        with _Surfer(model=self._model, part_id=part_id) as surfer:
            return surfer.create_shell_bl_using_controls(
                part_id, shell_bl_control_ids, shell_bl_params
            )

    def remesh_face_zonelets_locally(
        self,
        part_id: int,
        face_zonelets: Iterable[int],
        register_id: int,
        local_surfer_params: LocalSurferParams,
    ) -> LocalSurferResults:
        """Remesh face zonelets locally at the registered faces with given parameters.

        Parameters
        ----------
        part_id : int
            ID of part.
        face_zonelets : Iterable[int]
            IDs of the face zonelets.
        register_id : int
            Register ID of the target faces.
        local_surfer_params : LocalSurferParams
            Local surfer parameters.

        Returns
        -------
        LocalSurferResults
            Results from remeshing the face zonelets.

        Examples
        --------
        >>> surfer = prime.Surfer(model)
        >>> local_surfer_params = prime.LocalSurferParams(model)
        >>> results = surfer.remesh_face_zonelets_locally(part_id,
                        face_zonelets, register_id, local_surfer_params)

        """
        with _Surfer(model=self._model, part_id=part_id) as surfer:
            return surfer.remesh_face_zonelets_locally(
                face_zonelets, register_id, local_surfer_params
            )

    def remesh_face_zonelets(
        self,
        part_id: int,
        face_zonelets: Iterable[int],
        edge_zonelets: Iterable[int],
        params: SurferParams,
    ) -> SurferResults:
        """Perform meshing on face zonelets with given parameters.

        Parameters
        ----------
        part_id : int
            ID of part.
        face_zonelets : Iterable[int]
            IDs of the face zonelets.
        edge_zonelets : Iterable[int]
            IDs of the edge zonelets.
        params : SurferParams
            Surfer parameters.

        Returns
        -------
        SurferResults
            Results from meshing the face zonelets.

        Examples
        --------
        >>> surfer = prime.Surfer(model)
        >>> surfer_params = prime.SurferParams(model)
        >>> results = surfer.remesh_face_zonelets(part_id,
                        face_zonelets, edge_zonelets, surfer_params)

        """
        with _Surfer(model=self._model, part_id=part_id) as surfer:
            return surfer.remesh_face_zonelets(face_zonelets, edge_zonelets, params)

    def initialize_surfer_params_for_wrapper(self) -> SurferParams:
        """Initialize surfer parameters to mesh surfaces generated by the wrapper.

        Returns
        -------
        SurferParams
            Surfer parameters initialized for wrapper inputs.

        Examples
        --------
        >>> surfer = prime.Surfer(model)
        >>> surfer_params = surfer.initialize_surfer_params_for_wrapper()

        """
        with _Surfer(model=self._model, part_id=-1) as surfer:
            return surfer.initialize_surfer_params_for_wrapper()

    def __enter__(self):
        """Run initializing context manager."""
        return self

    def __exit__(self, type, value, traceback):
        """Run when closing the context manager."""
        pass



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""" Auto-generated file. DO NOT MODIFY """
import enum
from typing import Dict, Any, Union, List, Iterable
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.internals import utils
from ansys.meshing.prime.autogen.coreobject import *
import numpy as np

from ansys.meshing.prime.params.primestructs import *

class VolumeControlParams(CoreObject):
    """Volume control parameters are used to define the volume type in the volume control.

    Parameters
    ----------
    model: Model
        Model to create a ``VolumeControlParams`` object with default parameters.
    cell_zonelet_type: CellZoneletType, optional
        Cell zonelet type is used to define the type of the associated volume.
    skip_hexcore: bool, optional
        Check whether to skip hexahedral cells generation in the core for this volume or not. Applicable only for volumeFillType set to HexcoreTet or HexcorePoly in the AutoMeshParams structure.
    json_data: dict, optional
        JSON dictionary to create a ``VolumeControlParams`` object with provided parameters.

    Examples
    --------
    >>> volume_control_params = prime.VolumeControlParams(model = model)
    """
    _default_params = {}

    def __initialize(
            self,
            cell_zonelet_type: CellZoneletType,
            skip_hexcore: bool):
        self._cell_zonelet_type = CellZoneletType(cell_zonelet_type)
        self._skip_hexcore = skip_hexcore

    def __init__(
            self,
            model: CommunicationManager=None,
            cell_zonelet_type: CellZoneletType = None,
            skip_hexcore: bool = None,
            json_data : dict = None,
             **kwargs):
        """Initialize a ``VolumeControlParams`` object.

        Parameters
        ----------
        model: Model
            Model to create a ``VolumeControlParams`` object with default parameters.
        cell_zonelet_type: CellZoneletType, optional
            Cell zonelet type is used to define the type of the associated volume.
        skip_hexcore: bool, optional
            Check whether to skip hexahedral cells generation in the core for this volume or not. Applicable only for volumeFillType set to HexcoreTet or HexcorePoly in the AutoMeshParams structure.
        json_data: dict, optional
            JSON dictionary to create a ``VolumeControlParams`` object with provided parameters.

        Examples
        --------
        >>> volume_control_params = prime.VolumeControlParams(model = model)
        """
        if json_data:
            self.__initialize(
                CellZoneletType(json_data["cellZoneletType"] if "cellZoneletType" in json_data else None),
                json_data["skipHexcore"] if "skipHexcore" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [cell_zonelet_type, skip_hexcore])
            if all_field_specified:
                self.__initialize(
                    cell_zonelet_type,
                    skip_hexcore)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass a model or specify all properties.")
                else:
                    param_json = model._communicator.initialize_params(model, "VolumeControlParams")
                    json_data = param_json["VolumeControlParams"] if "VolumeControlParams" in param_json else {}
                    self.__initialize(
                        cell_zonelet_type if cell_zonelet_type is not None else ( VolumeControlParams._default_params["cell_zonelet_type"] if "cell_zonelet_type" in VolumeControlParams._default_params else CellZoneletType(json_data["cellZoneletType"] if "cellZoneletType" in json_data else None)),
                        skip_hexcore if skip_hexcore is not None else ( VolumeControlParams._default_params["skip_hexcore"] if "skip_hexcore" in VolumeControlParams._default_params else (json_data["skipHexcore"] if "skipHexcore" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            cell_zonelet_type: CellZoneletType = None,
            skip_hexcore: bool = None):
        """Set the default values of the ``VolumeControlParams`` object.

        Parameters
        ----------
        cell_zonelet_type: CellZoneletType, optional
            Cell zonelet type is used to define the type of the associated volume.
        skip_hexcore: bool, optional
            Check whether to skip hexahedral cells generation in the core for this volume or not. Applicable only for volumeFillType set to HexcoreTet or HexcorePoly in the AutoMeshParams structure.
        """
        args = locals()
        [VolumeControlParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ``VolumeControlParams`` object.

        Examples
        --------
        >>> VolumeControlParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in VolumeControlParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._cell_zonelet_type is not None:
            json_data["cellZoneletType"] = self._cell_zonelet_type
        if self._skip_hexcore is not None:
            json_data["skipHexcore"] = self._skip_hexcore
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "cell_zonelet_type :  %s\nskip_hexcore :  %s" % (self._cell_zonelet_type, self._skip_hexcore)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def cell_zonelet_type(self) -> CellZoneletType:
        """Cell zonelet type is used to define the type of the associated volume.
        """
        return self._cell_zonelet_type

    @cell_zonelet_type.setter
    def cell_zonelet_type(self, value: CellZoneletType):
        self._cell_zonelet_type = value

    @property
    def skip_hexcore(self) -> bool:
        """Check whether to skip hexahedral cells generation in the core for this volume or not. Applicable only for volumeFillType set to HexcoreTet or HexcorePoly in the AutoMeshParams structure.
        """
        return self._skip_hexcore

    @skip_hexcore.setter
    def skip_hexcore(self, value: bool):
        self._skip_hexcore = value



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

""" Auto-generated file. DO NOT MODIFY """
import enum
from typing import Dict, Any, Union, List, Iterable
from ansys.meshing.prime.internals.comm_manager import CommunicationManager
from ansys.meshing.prime.internals import utils
from ansys.meshing.prime.autogen.coreobject import *
import numpy as np

from ansys.meshing.prime.params.primestructs import *

class ShellBLControlGrowthParams(CoreObject):
    """Growth parameters for ShellBL control.

    Parameters
    ----------
    model: Model
        Model to create a ``ShellBLControlGrowthParams`` object with default parameters.
    n_layers: int, optional
        Number of layers to be generated.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    offset_type: ShellBLOffsetType, optional
        Offset type for ShellBL.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    growth_rate: float, optional
        Ratio of height of the current layer to the previous layer.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    first_height: float, optional
        Height of first layer of ShellBL.

        **This is a beta parameter**. **The behavior and name may change in the future**.
    json_data: dict, optional
        JSON dictionary to create a ``ShellBLControlGrowthParams`` object with provided parameters.

    Examples
    --------
    >>> shell_bl_control_growth_params = prime.ShellBLControlGrowthParams(model = model)
    """
    _default_params = {}

    def __initialize(
            self,
            n_layers: int,
            offset_type: ShellBLOffsetType,
            growth_rate: float,
            first_height: float):
        self._n_layers = n_layers
        self._offset_type = ShellBLOffsetType(offset_type)
        self._growth_rate = growth_rate
        self._first_height = first_height

    def __init__(
            self,
            model: CommunicationManager=None,
            n_layers: int = None,
            offset_type: ShellBLOffsetType = None,
            growth_rate: float = None,
            first_height: float = None,
            json_data : dict = None,
             **kwargs):
        """Initialize a ``ShellBLControlGrowthParams`` object.

        Parameters
        ----------
        model: Model
            Model to create a ``ShellBLControlGrowthParams`` object with default parameters.
        n_layers: int, optional
            Number of layers to be generated.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        offset_type: ShellBLOffsetType, optional
            Offset type for ShellBL.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        growth_rate: float, optional
            Ratio of height of the current layer to the previous layer.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        first_height: float, optional
            Height of first layer of ShellBL.

            **This is a beta parameter**. **The behavior and name may change in the future**.
        json_data: dict, optional
            JSON dictionary to create a ``ShellBLControlGrowthParams`` object with provided parameters.

        Examples
        --------
        >>> shell_bl_control_growth_params = prime.ShellBLControlGrowthParams(model = model)
        """
        if json_data:
            self.__initialize(
                json_data["nLayers"] if "nLayers" in json_data else None,
                ShellBLOffsetType(json_data["offsetType"] if "offsetType" in json_data else None),
                json_data["growthRate"] if "growthRate" in json_data else None,
                json_data["firstHeight"] if "firstHeight" in json_data else None)
        else:
            all_field_specified = all(arg is not None for arg in [n_layers, offset_type, growth_rate, first_height])
            if all_field_specified:
                self.__initialize(
                    n_layers,
                    offset_type,
                    growth_rate,
                    first_height)
            else:
                if model is None:
                    raise ValueError("Invalid assignment. Either pass a model or specify all properties.")
                else:
                    param_json = model._communicator.initialize_params(model, "ShellBLControlGrowthParams")
                    json_data = param_json["ShellBLControlGrowthParams"] if "ShellBLControlGrowthParams" in param_json else {}
                    self.__initialize(
                        n_layers if n_layers is not None else ( ShellBLControlGrowthParams._default_params["n_layers"] if "n_layers" in ShellBLControlGrowthParams._default_params else (json_data["nLayers"] if "nLayers" in json_data else None)),
                        offset_type if offset_type is not None else ( ShellBLControlGrowthParams._default_params["offset_type"] if "offset_type" in ShellBLControlGrowthParams._default_params else ShellBLOffsetType(json_data["offsetType"] if "offsetType" in json_data else None)),
                        growth_rate if growth_rate is not None else ( ShellBLControlGrowthParams._default_params["growth_rate"] if "growth_rate" in ShellBLControlGrowthParams._default_params else (json_data["growthRate"] if "growthRate" in json_data else None)),
                        first_height if first_height is not None else ( ShellBLControlGrowthParams._default_params["first_height"] if "first_height" in ShellBLControlGrowthParams._default_params else (json_data["firstHeight"] if "firstHeight" in json_data else None)))
        self._custom_params = kwargs
        if model is not None:
            [ model._logger.warning(f'Unsupported argument : {key}') for key in kwargs ]
        [setattr(type(self), key, property(lambda self, key = key:  self._custom_params[key] if key in self._custom_params else None,
        lambda self, value, key = key : self._custom_params.update({ key: value }))) for key in kwargs]
        self._freeze()

    @staticmethod
    def set_default(
            n_layers: int = None,
            offset_type: ShellBLOffsetType = None,
            growth_rate: float = None,
            first_height: float = None):
        """Set the default values of the ``ShellBLControlGrowthParams`` object.

        Parameters
        ----------
        n_layers: int, optional
            Number of layers to be generated.
        offset_type: ShellBLOffsetType, optional
            Offset type for ShellBL.
        growth_rate: float, optional
            Ratio of height of the current layer to the previous layer.
        first_height: float, optional
            Height of first layer of ShellBL.
        """
        args = locals()
        [ShellBLControlGrowthParams._default_params.update({ key: value }) for key, value in args.items() if value is not None]

    @staticmethod
    def print_default():
        """Print the default values of ``ShellBLControlGrowthParams`` object.

        Examples
        --------
        >>> ShellBLControlGrowthParams.print_default()
        """
        message = ""
        message += ''.join(str(key) + ' : ' + str(value) + '\n' for key, value in ShellBLControlGrowthParams._default_params.items())
        print(message)

    def _jsonify(self) -> Dict[str, Any]:
        json_data = {}
        if self._n_layers is not None:
            json_data["nLayers"] = self._n_layers
        if self._offset_type is not None:
            json_data["offsetType"] = self._offset_type
        if self._growth_rate is not None:
            json_data["growthRate"] = self._growth_rate
        if self._first_height is not None:
            json_data["firstHeight"] = self._first_height
        [ json_data.update({ utils.to_camel_case(key) : value }) for key, value in self._custom_params.items()]
        return json_data

    def __str__(self) -> str:
        message = "n_layers :  %s\noffset_type :  %s\ngrowth_rate :  %s\nfirst_height :  %s" % (self._n_layers, self._offset_type, self._growth_rate, self._first_height)
        message += ''.join('\n' + str(key) + ' : ' + str(value) for key, value in self._custom_params.items())
        return message

    @property
    def n_layers(self) -> int:
        """Number of layers to be generated.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._n_layers

    @n_layers.setter
    def n_layers(self, value: int):
        self._n_layers = value

    @property
    def offset_type(self) -> ShellBLOffsetType:
        """Offset type for ShellBL.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._offset_type

    @offset_type.setter
    def offset_type(self, value: ShellBLOffsetType):
        self._offset_type = value

    @property
    def growth_rate(self) -> float:
        """Ratio of height of the current layer to previous layer.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._growth_rate

    @growth_rate.setter
    def growth_rate(self, value: float):
        self._growth_rate = value

    @property
    def first_height(self) -> float:
        """Height of first layer of ShellBL.

        **This is a beta parameter**. **The behavior and name may change in the future**.
        """
        return self._first_height

    @first_height.setter
    def first_height(self, value: float):
        self._first_height = value



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Module containing PrismControl related classes and methods."""
from ansys.meshing.prime.autogen.prismcontrol import PrismControl as _PrismControl

# isort: split
from ansys.meshing.prime.autogen.commonstructs import SetNameResults


class PrismControl(_PrismControl):
    """PrismControl allows you to generate prisms.

    PrismControl allows you to control generation of prisms.
    Controls include setting the face scope, volume scope and growth parameters.

    Parameters
    ----------
    model : Model
        Server model to create PrismControl object.
    id : int
        Id of the PrismControl.
    object_id : int
        Object id of the PrismControl.
    name : str
        Name of the PrismControl.
    """

    def __init__(self, model, id, object_id, name, local=False):
        """Initialize class variables and the superclass."""
        _PrismControl.__init__(self, model, id, object_id, name)
        self._model = model
        self._name = name

    def set_suggested_name(self, name: str) -> SetNameResults:
        """Set the unique name for the prism control based on the suggested name.

        Parameters
        ----------
        name : str
            Suggested name for the prism control.

        Returns
        -------
        SetNameResults
            Returns the SetNameResults.


        Examples
        --------
        >>> prism_control.set_suggested_name("control1")

        """
        result = _PrismControl.set_suggested_name(self, name)
        self._name = result.assigned_name
        return result



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Module for logger singleton."""

# logger from https://gist.github.com/huklee/cea20761dd05da7c39120084f52fcc7c
import datetime
import logging
import os
from typing import Union

from ansys.meshing.prime.internals import utils


class SingletonType(type):
    """Provides the singleton helper class for the logger."""

    _instances = {}

    def __call__(cls, *args, **kwargs):
        """Call to redirect new instances to the singleton instance."""
        if cls not in cls._instances:
            cls._instances[cls] = super(SingletonType, cls).__call__(*args, **kwargs)
        return cls._instances[cls]


class PrimeLogger(object, metaclass=SingletonType):
    """Provides the singleton logger for PyPrimeMesh."""

    _logger = None

    def __init__(self, level: int = logging.WARNING, logger_name: str = "PyPrimeMesh"):
        """Logger initializer."""
        self._logger = logging.getLogger(logger_name)
        self._logger.setLevel(level)
        self._formatter = logging.Formatter(
            '%(asctime)s \t [%(levelname)s | %(filename)s:%(lineno)s] > %(message)s'
        )

    @property
    def python_logger(self) -> logging.Logger:
        """Get the python logger.

        Returns
        -------
        logging.Logger
            Logger.
        """
        return self._logger

    def set_level(self, level: Union[int, str]):
        """Set logger output level.

        Parameters
        ----------
        level : int
            Level of the logger.

        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.
        """
        utils.print_beta_api_warning(self._logger, "set_level")
        self._logger.setLevel(level=level)

    def enable_output(self, stream=None):
        """Enable logger output to given stream.

        If stream is not specified, sys.stderr is used.

        Parameters
        ----------
        stream: TextIO, optional
            Stream to output the log output to stream

        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.
        """
        # stdout
        utils.print_beta_api_warning(self._logger, "enable_output")
        stream_handler = logging.StreamHandler(stream)
        stream_handler.setFormatter(self._formatter)
        self._logger.addHandler(stream_handler)

    def add_file_handler(
        self, log_dir: str = "./.log", log_file: str = None
    ) -> logging.FileHandler:
        """Save logs to a file.

        Parameters
        ----------
        log_dir : str, optional
            Directory of the logs. The default is ``"./.log"``.

        log_file : str, optional
            Log filename. The default is ``"log_<datestamp>.log"``.

        Notes
        -----
        **This is a beta API**. **The behavior and implementation may change in future**.
        """
        utils.print_beta_api_warning(self._logger, "add_file_handler")
        if log_file is None:
            now = datetime.datetime.now()
            log_file = "log_" + now.strftime("%Y-%m-%d") + ".log"
        if not os.path.isdir(log_dir):
            os.mkdir(log_dir)
        file_handler = logging.FileHandler(os.path.join(log_dir, log_file))
        file_handler.setFormatter(self._formatter)
        self._logger.addHandler(file_handler)
        return file_handler



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Module containing VolumeControl related classes and methods."""
from ansys.meshing.prime.autogen.volumecontrol import VolumeControl as _VolumeControl

# isort: split
from ansys.meshing.prime.autogen.commonstructs import SetNameResults
from ansys.meshing.prime.autogen.volumecontrolstructs import VolumeControlParams


class VolumeControl(_VolumeControl):
    """Defines the scope and type of volume mesh to generate.

    Parameters
    ----------
    model : Model
        Server model to create VolumeControl object.
    id : int
        Id of the VolumeControl.
    object_id : int
        Object id of the VolumeControl.
    name : str
        Name of the VolumeControl.
    """

    def __init__(self, model, id, object_id, name, local=False):
        """Initialize the superclass and the ``model`` variable."""
        _VolumeControl.__init__(self, model, id, object_id, name)
        self._model = model
        self._name = name

    def __str__(self) -> str:
        """Get a representation of the class in string format.

        Returns
        -------
        str
            Class data in string format.
        """
        params = VolumeControlParams(model=self._model)
        # TODO: function get_summary() not implemented
        result = _VolumeControl.get_summary(self, params)
        return result.message

    def set_suggested_name(self, name: str) -> SetNameResults:
        """Set the unique name for the volume control based on a suggested name.

        Parameters
        ----------
        name : str
            Suggested name for the volume control.

        Returns
        -------
        SetNameResults
            Newly suggested name of the volume control.


        Examples
        --------
        >>> volume_control.set_suggested_name("control1")

        """
        result = _VolumeControl.set_suggested_name(self, name)
        self._name = result.assigned_name
        return result

    @property
    def name(self):
        """Get the name of the volume control.

        Returns
        -------
        str
            Name of the volume control.

        Examples
        --------
        >>> print(volume_control.name)

        """
        return self._name
